local L = LibStub("AceLocale-3.0"):GetLocale("LootersAid")
local testmode = false
local GetSpellInfo = GetSpellInfo
local GetSpellLink = GetSpellLink
local msg, growl, torment, thunderstomp, petmode
local growlname, tormentname, thunderstompname, sufferingname
local pack, cheetah, ptext, ctext
local items = {}
local _G = _G
local GetInventoryItemLink = GetInventoryItemLink
local sfind = string.find

local function IsInstance()

	local inInstance, instanceType = IsInInstance()
	local ininstance = false

	if inInstance then

		if instanceType ~= "none" then ininstance = true end

	end

  	return ininstance

end

local function fadeOut(self, elapsed)

	local fadetime = Kipper.frame.fadetime + elapsed
	
	if Kipper.frame.fadetime == 0 then
	
		PlaySound("RaidWarning")
		
	end
	
	if fadetime < 2 then

		Kipper.frame.fadetime = fadetime

	else

		local fadeInfo = {}
		
		Kipper.frame:SetScript("OnUpdate", nil)
		Kipper.frame.fadetime = 0
		fadeInfo.mode = "OUT"
		fadeInfo.timeToFade = 4
		fadeInfo.startAlpha = 1
		fadeInfo.endAlpha = 0
		fadeInfo.finishedFunc = 
			function()
				Kipper.frame:Hide()
				Kipper.frame:SetAlpha(1)
			end
			
		UIFrameFade(Kipper.frame, fadeInfo)
	
	end
	
end

local function showMsgs()
	
	for n, item in ipairs(items) do _G["Kipper_Msg" .. n]:SetText(item) end
	
	Kipper.frame:Show()
	Kipper.frame:SetScript("OnUpdate", fadeOut)
	
end

local function addMsg(link, linktype)

	if #items < 5 then
	
		if linktype == 2 then
		
			table.insert(items, L["Warning!"] .. " " .. link .." " .. L["missing!"])
			
		elseif linktype == 3 then
		
			table.insert(items, L["Warning!"] .. " " .. link .." " .. L["enabled!"])
			
		else
		
			table.insert(items, L["Warning!"] .. " " .. link .." " .. L["equipped!"])
			
		end
		
	end
	
end

local function GetPetMode()

	local mode
	local actionslot, ret
	
	if HasPetSpells() then
	
		for actionslot = 1, 10 do
	
			local name, _, _, _, active = GetPetActionInfo(actionslot)
		
			if active then
		
				if string.sub(name,1, 8) == "PET_MODE" then
			
					if name == "PET_MODE_PASSIVE" then
				
						mode = _G[name]
						break
						
					end
				
				end
			
			end
	
		end
	
	end

	return mode
	
end

local function checkBuffs()
	
	local ret
	
	if Kipper.CLASS == "HUNTER" then
	
		--check for Aspect of the Pack or Aspect of the Cheetah
		if UnitBuff("player", ptext) then

			addMsg(pack, 3)
			ret = 1
			
		elseif UnitBuff("player", ctext) then
		
			addMsg(cheetah, 3)
			ret = 1
			
		end
		
	end
	
	return ret
	
end

local function checkGrowl()

	local auto, ret
	
	if Kipper.CLASS == "HUNTER" then
	
		auto = select(2, GetSpellAutocast(growlname, BOOKTYPE_PET))
		
		if auto then
		
			addMsg(growl, 3)
			ret = 1
			
		end
		
		auto = select(2, GetSpellAutocast(thunderstompname, BOOKTYPE_PET))
		
		if auto then
		
			addMsg(thunderstomp, 3)
			ret = 1
			
		end
		
	elseif Kipper.CLASS == "WARLOCK" then
	
		auto = select(2, GetSpellAutocast(tormentname, BOOKTYPE_PET))
		
		if auto then
		
			addMsg(torment, 3)
			ret = 1
			
		end
		
		auto = select(2, GetSpellAutocast(sufferingname, BOOKTYPE_PET))
		
		if auto then
		
			addMsg(torment, 3)
			ret = 1
			
		end
		
	end
	
	return ret
	
end

local function checkGear()

	local slot, itemid, ench
	local ilink, i
	local found, checkitem, custom
	local result
	local hasaura, trank
	local foundb
	
	found = nil
	result = 1

	Kipper.frame:SetScript("OnUpdate", nil)
	
	--check inventory
	for slot = 0, 16 do

		ilink = GetInventoryItemLink("Player", slot)

		if ilink then

			_, _, _, _, itemid, ench = sfind(ilink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")


			for i, checkitem in ipairs(Kipper.checkitems) do

				result = itemid - checkitem

				if result == 0 then

					found = 1
					addMsg(ilink)
	
				end
	
			end

			for i, enchant in pairs(Kipper.enchantids) do

				result = ench - enchant

				if result == 0 then

					found = 1
					addMsg(ilink)
	
				end

			end

			if #KipperDB.custom > 0 then
			
				for i, custom in pairs(KipperDB.custom) do
				
					result = itemid - custom
					
					if result == 0 then
					
						found = 1
						addMsg(ilink)
						
					end
					
				end
				
			end

		end

	end

	if checkGrowl() then
	
		found = 1
		
	end
	
	petmode = GetPetMode()
	
	if petmode then
	
		addMsg("|cff71d5ff[" .. _G.PET .. " " .. petmode .. "]|r", 3)
		found = 1
		
	end
	
	if checkBuffs() then
	
		found = 1
		
	end
	
	if found then

		showMsgs()

	else
	
		Kipper.frame:Hide()
		
	end		

end

local function delay(self, elapsed)

	local waittime = Kipper.frame.wait + elapsed

	if waittime < 5 then

		Kipper.frame.wait = waittime

	else

		checkGear()

	end
	
end

function Kipper:showHelp()

	local f = DEFAULT_CHAT_FRAME
	local c1, c2 = "|cff4682b4", "|cffeedd82"

	f:AddMessage("Kipper slash commands", 0, 1, 1)
	f:AddMessage("Usage: /kipper { add | remove }", 0, 1 ,1)
	f:AddMessage(" - add " .. c1 .. "<item id / item link>" .. c2 .. ": Add a custom item", 0, 1, 1)
	f:AddMessage(" - remove " .. c1 .. "<item id / item link>" .. c2 .. ": Remove a custom item", 0, 1, 1)

end

local function isLink(val)

	itemid = string.match(val, "item:(%d+)")
	return tonumber(itemid)
	
end

function Kipper:chatCommand(input)

	local command = string.lower(input or "")
	local space = string.find(command, " ")
	local nextspace, itemid, i, v
	local params = ""
	local valid, invalid
	
	if space then

		params = string.lower(string.sub(command, space + 1))
		command = string.sub(command, 1, space - 1)

	end
	
	if command == "add" then
	
		if params then

			itemid = isLink(params) or tonumber(params)

			if itemid then
				
				if #KipperDB.custom > 0 then
					
					for i,v in pairs(KipperDB.custom) do

						if v == itemid then

							invalid = true
							break

						end
							
					end
						
					if not invalid then
						
						table.insert(KipperDB.custom, itemid)
						DEFAULT_CHAT_FRAME:AddMessage("Kipper: Added item id " .. itemid, 0, 1, 1)
						valid = true

					end
					
				else
					
					KipperDB.custom = {itemid}
					DEFAULT_CHAT_FRAME:AddMessage("Kipper: Added item id " .. itemid, 0, 1, 1)
					valid = true
					
				end
				
			end
				
		end
		
	elseif command == "remove" then
	
		if params then
		
			itemid = isLink(params) or tonumber(params)
			
			if itemid then
				
				if #KipperDB.custom > 0 then
					
					for i,v in pairs(KipperDB.custom) do
						
						if v == itemid then
							
							invalid = i
							break

						end
							
					end
						
					if invalid > 0 then
						
						table.remove(KipperDB.custom, invalid)
						DEFAULT_CHAT_FRAME:AddMessage("Kipper: Removed item id " .. itemid, 0, 1, 1)
						valid = true

					end

				end
					
			end
				
		end
		
	end
	
	if not valid then
	
		Kipper:showHelp()
		
	end
	
end

local function eventHandler(self, event, arg1, ...)

	if ((event == "ADDON_LOADED") and (arg1 == "Kipper")) then

		if not KipperDB then
		
			KipperDB = {}
			
		end
		
		if not KipperDB.custom then
		
			KipperDB.custom = {}
			
		end
		
		SLASH_KIPPER_CMD1 = "/kipper"
		SlashCmdList["KIPPER_CMD"] = function(input) Kipper:chatCommand(input) end
		
	elseif event == "PLAYER_ENTERING_WORLD" then

		Kipper.frame:Hide()
		
		if IsInstance() then

			Kipper_Msg1:SetText("")
			Kipper_Msg2:SetText("")
			Kipper_Msg3:SetText("")
			Kipper_Msg4:SetText("")
			Kipper_Msg5:SetText("")
			items = {}
			--Kipper.ROLE = UnitGroupRolesAssigned("player")
			--if ((Kipper.ROLE == "NONE") and (testmode == true)) then Kipper.ROLE = "DAMAGER" end
			Kipper.frame.wait = 0
			Kipper.frame:Show()
			Kipper.frame:SetScript("OnUpdate", delay)
			
		end

	elseif event == "PLAYER_LOGIN" then

		if not msg then

			msg1 = Kipper.frame:CreateFontString("Kipper_Msg1", "ARTWORK", "GameFontNormalLarge")
			msg1:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg1:SetPoint("CENTER", Kipper.frame, "CENTER", 0, 0)

			msg2 = Kipper.frame:CreateFontString("Kipper_Msg2", "ARTWORK", "GameFontNormalLarge")
			msg2:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg2:SetPoint("CENTER", msg1, "CENTER", 0, 32)
			
			msg3 = Kipper.frame:CreateFontString("Kipper_Msg3", "ARTWORK", "GameFontNormalLarge")
			msg3:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg3:SetPoint("CENTER", msg2, "CENTER", 0, 32)
			
			msg4 = Kipper.frame:CreateFontString("Kipper_Msg4", "ARTWORK", "GameFontNormalLarge")
			msg4:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg4:SetPoint("CENTER", msg3, "CENTER", 0, 32)
			
			msg5 = Kipper.frame:CreateFontString("Kipper_Msg5", "ARTWORK", "GameFontNormalLarge")
			msg5:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg5:SetPoint("CENTER", msg4, "CENTER", 0, 32)
			
			growl = GetSpellLink(2649)
			growlname = GetSpellInfo(2649)
			torment = GetSpellLink(3716)
			tormentname = GetSpellInfo(3716)
			thunderstomp = GetSpellLink(63900)
			thunderstompname = GetSpellInfo(63900)
			sufferingname = GetSpellInfo(17735)
			pack = GetSpellLink(13159)
			ptext = GetSpellInfo(13159)
			cheetah = GetSpellLink(5118)
			ctext = GetSpellInfo(5118)

			_, Kipper.CLASS = UnitClass("player")
			--Kipper.LEVEL = UnitLevel("player")

		end

	end

end

Kipper.frame = CreateFrame("Frame","Kipper",UIParent)
Kipper.frame:Hide()
Kipper.frame.fadetime = 0
Kipper.frame.wait = 0
Kipper.frame:SetFrameStrata("DIALOG")
Kipper.frame:SetHeight(200)
Kipper.frame:SetWidth(UIParent:GetWidth())
Kipper.frame:SetPoint("CENTER", UIParent, "CENTER", 0, 150)
Kipper.frame:RegisterEvent("ADDON_LOADED")
Kipper.frame:RegisterEvent("PLAYER_LOGIN")
Kipper.frame:RegisterEvent("PLAYER_ENTERING_WORLD")
Kipper.frame:SetScript("OnEvent", eventHandler)